//
// client.cpp
// ~~~~~~~~~~
//
// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#include <boost/asio.hpp>
#include <boost/log/attributes.hpp>
#include <boost/log/utility/setup/common_attributes.hpp>
#include <boost/log/core.hpp>
// #include <boost/log/sinks/text_file_backend.hpp>
#include <boost/log/trivial.hpp>
#include <boost/log/utility/setup/file.hpp>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <map>
#include <string>

#include "config.h"
#include "session.h"

void init_logging(void) {
  /*
    std::ifstream file("log_settings.ini");
    if (!file.is_open()) {
      std::cerr << "Could not open log_settings.ini file." << std::endl;
      exit(1);
    }

    boost::log::init_from_stream(file);
    */
  // boost::log::add_file_log("proxy.log"); // WAT?
  // boost::shared_ptr< boost::log::core > core = boost::log::core::get();
  // core->add_global_attribute("TimeStamp", boost::attrs::local_clock());
  boost::log::add_common_attributes();

  boost::log::add_file_log(
    boost::log::keywords::file_name = "proxy.log",
    boost::log::keywords::format = "[%TimeStamp%]: %Message%"
  );
  boost::log::core::get()->set_filter(boost::log::trivial::severity >=
                                      boost::log::trivial::info);
}

int main(int argc, char *argv[]) {
  // boost::json::json_value config;

  if (argc != 2) {
    std::cerr << "Usage: twproxy <filename>" << std::endl;
    return EXIT_FAILURE;
  }

  init_logging();

  std::map<std::string, std::string> config = yaml_parse(argv[1]);

  /*
  try {
    // Parse the file as JSON
    config = yaml_parse( argv[1] );

  } catch (std::exception const &e) {
    std::cerr << "Caught exception: " << e.what() << std::endl;
    return EXIT_FAILURE;
  }
  */

  bool config_ok = true;

  // for (const char *key : {"server", "host", "port"}) {
  for (auto key : {"server", "host", "port"}) {
    auto pos = config.find(key);
    if (pos == config.end()) {
      config_ok = false;
      std::cout << "Config file missing: " << key << std::endl;
      BOOST_LOG_TRIVIAL(fatal) << "Config file missing: " << key;
    }
    BOOST_LOG_TRIVIAL(info) << "Config: " << key << " : " << config[key];
  }
  if (!config_ok)
    return 2;

  int port = std::stoi(config["server"]);
  // int port = 9999; // 2002;

  try {

    boost::asio::io_service io_service;

    boost::asio::ip::tcp::endpoint endpoint(boost::asio::ip::tcp::v4(),
                                            port); // std::atoi(argv[i]));
    // connect to the BBS
    server s(io_service, endpoint, config["host"], config["port"]);
    //"127.0.0.1", "2023");

    io_service.run();
  } catch (std::exception &e) {
    std::cerr << "Exception: " << e.what() << "\n";
  }

  return EXIT_SUCCESS;
}
